package ru.dbotthepony.mc.otm.screen.panels;

import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.client.gui.narration.NarratableEntry;
import net.minecraft.client.gui.narration.NarrationElementOutput;
import net.minecraft.network.chat.Component;
import ru.dbotthepony.mc.otm.screen.MatteryScreen;
import ru.dbotthepony.mc.otm.screen.RenderHelper;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

public class FramePanel extends EditablePanel implements NarratableEntry {
	protected Component title;

	public FramePanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, float x, float y, float width, float height, Component title) {
		super(screen, parent, x, y, width, height);
		this.title = title;
		setDockPadding(4, 14, 4, 4);
	}

	protected boolean dragging = false;
	protected double last_mouse_x = 0;
	protected double last_mouse_y = 0;

	@Override
	protected boolean mouseClickedInner(double mouse_x, double mouse_y, int flag) {
		var pos = screenToLocal(mouse_x, mouse_y);

		if (getParent() == null && pos.y() >= 0 && pos.y() <= 12) {
			setIgnoreMouseEventBoundaries(true);
			last_mouse_x = mouse_x;
			last_mouse_y = mouse_y;
			dragging = true;
			return true;
		}

		return super.mouseClickedInner(mouse_x, mouse_y, flag);
	}

	@Override
	protected boolean mouseReleasedInner(double mouse_x, double mouse_y, int flag) {
		if (dragging) {
			setIgnoreMouseEventBoundaries(false);

			dragging = false;
			return true;
		}

		return super.mouseReleasedInner(mouse_x, mouse_y, flag);
	}

	@Override
	protected boolean mouseDraggedInner(double mouse_x, double mouse_y, int flag, double drag_x, double drag_y) {
		if (dragging) {
			var real_drag_x = mouse_x - last_mouse_x;
			var real_drag_y = mouse_y - last_mouse_y;

			last_mouse_x = mouse_x;
			last_mouse_y = mouse_y;

			setPos(getX() + real_drag_x, getY() + real_drag_y);
		}

		return super.mouseDraggedInner(mouse_x, mouse_y, flag, drag_x, drag_y);
	}

	@Override
	protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
		// background
		// RenderSystem.disableDepthTest();
		RenderHelper.drawWindowBackground(stack, 0, 0, getWidth(), getHeight());
		// RenderSystem.enableDepthTest();

		// title
		screen.getFont().draw(stack, title, 8, 5, 4210752);
	}

	@Override
	public NarrationPriority narrationPriority() {
		return NarrationPriority.NONE;
	}

	@Override
	public void updateNarration(NarrationElementOutput narrationElementOutput) {

	}
}
