package ru.dbotthepony.mc.otm.screen.panels;

import com.mojang.blaze3d.platform.InputConstants;
import com.mojang.blaze3d.systems.RenderSystem;
import com.mojang.blaze3d.vertex.PoseStack;
import net.minecraft.ChatFormatting;
import net.minecraft.Util;
import net.minecraft.client.Minecraft;
import net.minecraft.client.renderer.GameRenderer;
import net.minecraft.world.inventory.AbstractContainerMenu;
import net.minecraft.world.inventory.ClickType;
import net.minecraft.world.item.ItemStack;
import ru.dbotthepony.mc.otm.OverdriveThatMatters;
import ru.dbotthepony.mc.otm.menu.slot.MatterySlot;
import ru.dbotthepony.mc.otm.screen.MatteryScreen;
import ru.dbotthepony.mc.otm.screen.RGBAColor;
import ru.dbotthepony.mc.otm.screen.RenderHelper;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;

public class SlotPanel<T extends MatterySlot> extends EditablePanel {
	public final T slot;

	public SlotPanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, float x, float y, float width, float height, T slot) {
		super(screen, parent, x, y, width, height);
		this.slot = slot;
	}

	public SlotPanel(@Nonnull MatteryScreen<?> screen, @Nullable EditablePanel parent, T slot) {
		this(screen, parent, slot.x, slot.y, 18, 18, slot);
	}

	public final RGBAColor SLOT_HIGHLIGHT = new RGBAColor(255, 255, 255, 100);
	public final RGBAColor SLOT_HIGHLIGHT_DRAG = new RGBAColor(200, 200, 200, 150);

	@Override
	protected boolean mouseClickedInner(double mouse_x, double mouse_y, int button) {
		screen.onSlotClicked(slot, button);
		return true;
	}

	@Override
	protected boolean mouseDraggedInner(double mouse_x, double mouse_y, int mouse_code, double drag_x, double drag_y) {
		screen.onSlotMouseDragged(slot, mouse_code);
		return true;
	}

	@Override
	protected boolean mouseReleasedInner(double mouse_x, double mouse_y, int flag) {
		screen.onMouseReleasedSlot(slot, mouse_x, mouse_y, flag, false);
		return true;
	}

	protected void renderSlotBackground(PoseStack stack, float mouse_x, float mouse_y, float flag) {
		RenderHelper.drawRegularSlot(stack, 0, 0);
	}

	protected void renderRegular(PoseStack stack, ItemStack itemstack, String count_override) {
		if (!itemstack.isEmpty()) {
			RenderSystem.enableDepthTest();

			var system_stack = RenderSystem.getModelViewStack();

			system_stack.pushPose();
			system_stack.translate(parent_x + 1, parent_y + 1, 0);
			RenderSystem.applyModelViewMatrix();

			// Thanks Mojang
			// Very cool
			// (for int x, int y, which are then cast into doubles anyway)
			screen.getItemRenderer().blitOffset = accumulated_depth - 100; // force item to draw only 50 units "above" background
			screen.getItemRenderer().renderAndDecorateItem(Minecraft.getInstance().player, itemstack, 0, 0, (int) (parent_x + parent_y * 1000) /* some kind of indexing ? */);
			screen.getItemRenderer().renderGuiItemDecorations(screen.getFont(), itemstack, 0, 0, count_override);

			// too big accumulations can lead to Z near clipping issues
			accumulate3DHeight(100);

			system_stack.popPose();
			RenderSystem.applyModelViewMatrix();
		}

		if (is_hovered) {
			screen.hoveredSlot = slot;

			stack.pushPose();
			stack.translate(0, 0, 101);
			RenderHelper.setDrawColor(SLOT_HIGHLIGHT);
			RenderHelper.drawRect(stack, 1, 1, getWidth() - 1, getHeight() - 1);
			stack.popPose();
		}
	}

	@Override
	protected void innerRender(PoseStack stack, float mouse_x, float mouse_y, float flag) {
		renderSlotBackground(stack, mouse_x, mouse_y, flag);

		var itemstack = slot.getItem();
		var carried = screen.menu.getCarried();
		var drag_hit = false;
		String count_override = null;

		if (!carried.isEmpty() && screen.isQuickCrafting() && screen.quickCraftSlots.contains(slot)) {
			if (screen.quickCraftSlots.size() == 1) {
				return;
			}

			if (AbstractContainerMenu.canItemQuickReplace(slot, carried, true) && screen.menu.canDragTo(slot)) {
				itemstack = carried.copy();
				drag_hit = true;

				AbstractContainerMenu.getQuickCraftSlotCount(screen.quickCraftSlots, screen.getQuickCraftingType(), itemstack, slot.getItem().isEmpty() ? 0 : slot.getItem().getCount());
				int k = Math.min(itemstack.getMaxStackSize(), slot.getMaxStackSize(itemstack));

				if (itemstack.getCount() > k) {
					count_override = ChatFormatting.YELLOW.toString() + k;
					itemstack.setCount(k);
				}
			} else {
				screen.quickCraftSlots.remove(slot);
				screen.recalculateQuickCraftRemaining();
			}
		}

		RenderSystem.setShader(GameRenderer::getPositionTexShader);

		if (drag_hit) {
			RenderHelper.setDrawColor(SLOT_HIGHLIGHT_DRAG);
			RenderHelper.drawRect(stack, 1, 1, getWidth() - 1, getHeight() - 1);
		}

		renderRegular(stack, itemstack, count_override);
	}
}
